# Highcharts Spectrum

Веб приложение для отображения данных TANGO-атрибутов типа `spectrum` с перерисовкой или `scalar` с добавлением данных с помощью библиотеки `highcharts`. Для получения данных из атрибутов используется TANGO-модуль `WebSocketDS_cli` версии >= `1.0.0` (клиентная версия). Также можно использовать серверную версию, в этом случае атрибуты указанные в URL должны быть из списка прослушиваемых.

- **[настройки в conf файле](#conf-file)**
  - **[конфигурация websocket](#конфигурация-websocket)**
  - **[конфигурация hdbpp](#конфигурация-hdbpp)**

- **[использование](#использование)**

- **[get параметры](#get-параметры)**
  - **[get параметры основные](#get-параметры-основные)**
  - **[get параметры дополнительные](#get-параметры-дополнительные)**
  - **[get параметры дополнительные для типа данных](#get-параметры-дополнительные-для-типа-данных)**
  - **[get параметры дополнительные для настройки сигнала](#get-параметры-дополнительные-для-настройки-сигнала)**
  - **[get параметры дополнительные для настроек опций](#get-параметры-дополнительные-для-настроек-опций)**
  - **[get параметры дополнительные для hdbpp](#get-параметры-дополнительные-для-hdbpp)**

- **[типы данных](#типы-данных)**

- **[test mode](#test-mode)**

## conf file

Конф. файл `./assets/sett/common_sett.json`

Также можно использовать разные файлы настроек, для этого в URL параметре нужно указать постфикс для необходимого файла. Подробнее [ниже](#get-параметры-основные) параметр `commsett`.

### конфигурация websocket

В конф. файле `./assets/sett/common_sett.json` должны быть указаны настройки для подключения по websocket.

Пример:

```json
{
  "websocket": {
    "ws_url": "ws://localhost:12346"
  }
}
```

- **websocket.ws_url** - Если указывается полный адрес
- **websocket.ws_path** - Если указывается относительный путь
- **websocket.is_server_mode** - `true`, если используется серверный тип. По умолчанию `false` (boolean)

### конфигурация hdbpp

Для конфигурации hdbpp используется объект `hdbpp`

- **hdbpp.extractor_dev** - TANGO-device используемый для извлечения данных из БД hdbpp
- **hdbpp.login** - login для выполнения команды извлечения данных за указынный период
- **hdbpp.password** - password для выполнения команды извлечения данных за указынный период

## использование

Все настройки для графика полностью задаются пользователем в отдельном js-файле. Заданный график выводится во всю страницу.

JS-файл с настройками для объекта `Chart` библиотеки `highcharts` должен располагаться в директории `./assets/scripts/`.

Формат имени файла: `highchartjs_${NAME}.js`. Постфикс `${NAME}` передаётся GET параметром `chartscript`. Данный файл обязательно должен содержать переменную `var hcscript`, объявляющуюю объект `Options` для `Chart`. Подробнее про формат `Options` [здесь](https://api.highcharts.com/highcharts/chart)

Пример:

```javascript
var hcscript = {
  chart: {
    type: 'line',
    backgroundColor: 'black'
  },
  title: undefined,
  credits: {
    enabled: false
  },
  series: [
    {
      color: 'yellow',
      type: 'spline',
      enableMouseTracking: false,
    },
    {
      color: 'white',
      type: 'spline',
      enableMouseTracking: false,
    }
  ]
};
```

## get параметры

Приложение будет работать только при указании необходимых GET параметров.

### get параметры основные

Для отображения данных с конкретных TANGO-атрибутов типа `spectrum`, указываются следующие GET параметры:

- **dev** - Имя TANGO-device. Допускается только одно имя.
- **attrs** - Список атрибутов, через запятую. Идёт подписка на `change` события.
- **eventtype** - Тип события, на которое идёт подписка

Для указания имени файла настроек, если оно отличается от считываемого по умолчанию `assets/sett/common-sett.json`, используется следующий параметр

- **commsett** - Постфикс для используемого файла настроек. Имя файла должно быть в следующем формате `assets/sett/common-sett_${commsett}.json`

### get параметры дополнительные

Дополнительные GET параметры:

- **chartscript** - Постфикс для имени скрипта `/assets/scripts/highcharts_${chartscript}.js`. Этот файл должен содержать объект `Options` для `Chart`. Подробнее [выше](#использование).
- **testmode** - Если указан этот ключ, работает в тестовом режиме. Загружаются не данные из сервера, а случайные значения.
- **max_points** - Максимальное количество точек. Используется в режимах `scalar` и `hdbpp_scalar_upd`. После достижения максимума точки стираются с поздних с каждой операцией добавления новых данных
- **htmltitle** - Заголовок страницы в браузере. По умолчанию `chart`

### get параметры дополнительные для типа данных

- **procdatatype** - Тип получения и обработки данных. По умолчанию spectrum. Про типы [ниже](#типы-данных)

### get параметры дополнительные для настройки сигнала

Дополнительные GET параметры для сигнала о пришедших данных. Используется SVG circle

- **notusesignal** - Не использовать сигнал о получении данных с событий. По умолчанию используется
- **xpossignal** - Центр SVG circle. Позиция по X. По умолчанию 30
- **ypossignal** - Центр SVG circle. Позиция по Y. По умолчанию 30
- **radiussignal** - Радиус SVG circle. По умолчанию 20
- **colorsignal** - Цвет SVG circle. По умолчанию 'green'
- **timeoutsignal** - Время в миллисекундах на которое высвечивается SVG circle. По умолчанию 200.

### get параметры дополнительные для настроек опций

Дополнительные GET параметры для настроек `Chart.Options`:

- **aliases** - Список псевдонимов для атрибутов. Иначе отображаются имена атрибутов.
- **chmin** - Установка минимума для YAxis. (number) По умолчанию динамически
- **chmax** - Установка максимума для YAxis. (number) По умолчанию динамически
- **ytitle** - Текст заголовка для Y. Если не указаны настройки для заголовка выставляется `fontSize: '15px'`

### get параметры дополнительные для hdbpp

- **hdbpp_time_in_min** - Период времени, за который загружается данные, в минутах. По умолчанию 300
- **hdbpp_n_points** - Число точек. По умолчанию 300

## типы данных

Тип обрабатываемых и выводимых данных в графиках.

От ENUM_PROC_DATA_TYPE зависит шкала абцисс и процесс обработки и вывода данных

Тип выводится в ключе `procdatatype`. Подробнее [выше](#get-параметры-дополнительные-для-типа-данных)

Типы:

- **spectrum** - Простые данные из TANGO атрибутов типа SPECTRUM. График перерисовывается после каждого обновления данных. (Используется по умолчанию)
- **scalar** - Данные типа SCALAR. В график добавляются данные из событий. По шкале абцисс время. Максимальное количество данных (точек) для одного атрибута задаётся URL параметром ([max_points](#get-параметры-дополнительные)), либо выставляется значение по умолчанию (300)
- **hdbpp_scalar_upd** - Данные типа SCALAR. Выводятся данные за указанный последний период времени. В график добавляются данные из событий. По шкале абцисс время. Максимальное количество данных (точек) для одного атрибута задаётся URL параметром ([max_points](#get-параметры-дополнительные)), либо выставляется значение по умолчанию (300). Про возможные параметры для этого режима [выше](#get-параметры-дополнительные-для-hdbpp)

## test mode

При разработке, в режиме `ng serve` или `ng build --watch` можно тестировать настройки для `Options` в файле `src/app/services/fortest/testchart.ts`. Отслеживание изменений в git этого файла отключено.
Также не должен быть задан GET-параметр `chartscript`, указывающий скрипт для загрузки.

По умолчанию в тестовом режиме используется данный конфиг для `Chart.Options`:

```typescript
export const testChart: Options = {
  chart: {
    type: 'line',
    backgroundColor: 'black'
  },
  title: {
    text: 'TEST',
    align: 'center',
    style: {
      fontSize: '20',
      color: 'yellow'
    }
  },
  credits: {
    enabled: false
  },
  series: [
    {
      type: 'spline',
      color: 'yellow',
      enableMouseTracking: false,
    }
  ]
}
```
